@@ -2,8 +2,7 @@
 /*
  * Syscalls for uClibc
  *
- * Copyright (C) 2000 by Lineo, inc
- * Copyright (C) 2001, 2002 by Erik Andersen
+ * Copyright (C) 2001-2003 by Erik Andersen
  * Written by Erik Andersen <andersen@codpoet.org>
  *
  * This program is free software; you can redistribute it and/or modify it
@@ -47,14 +46,14 @@
 #	ifdef __UCLIBC_HAS_MMU__
 #define __NR___libc_fork __NR_fork
 		_syscall0(pid_t, __libc_fork);
-		weak_alias (__libc_fork, fork)
 #	else
-		pid_t fork(void)
+		pid_t __libc_fork(void)
 		{
 			__set_errno(ENOSYS);
 			return -1;
 		}
 #	endif
+weak_alias (__libc_fork, fork)
 #endif
 
 //#define __NR_read             3
@@ -79,16 +78,38 @@ weak_alias (__libc_write, __write)
 //#define __NR_open             5
 #ifdef L___syscall_open
 #define __NR___syscall_open __NR_open
+#include <stdlib.h>
 #include <stdarg.h>
-/* Do not include fcntl.h, so gcc doesn't whine the prototype */
+#include <fcntl.h>
+#include <string.h>
+#include <sys/param.h>
 static inline
-_syscall3(int, __syscall_open, const char *, fn, int, flags, __kernel_mode_t, mode);
-int __libc_open (const char * fn, int flags, mode_t mode)
+_syscall3(int, __syscall_open, const char *, file, int, flags, __kernel_mode_t, mode);
+int __libc_open (const char * file, int flags, ...)
 {
-	  return __syscall_open(fn, flags, mode);
+	mode_t mode;
+	if (unlikely(file==NULL || *file=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(file)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	if (flags & O_CREAT) {
+		va_list ap;
+		va_start(ap, flags);
+		mode = va_arg(ap, mode_t);
+		va_end(ap);
+	}
+	return __syscall_open(file, flags, mode);
 }
-
 weak_alias(__libc_open, open)
+
+int creat(const char *file, mode_t mode)
+{
+	  return __libc_open (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
+}
 #endif
 
 //#define __NR_close            6
@@ -103,23 +124,7 @@ weak_alias(__libc_close, close)
 // Implemented using wait4 
 
 //#define __NR_creat            8
-#ifdef L___syscall_creat
-#include <fcntl.h>
-#ifdef __NR_creat
-#define __NR___syscall_creat __NR_creat
-_syscall2(int, __syscall_creat, const char *, file, __kernel_mode_t, mode);
-int creat (const char *file, mode_t mode)
-{
-	  return __syscall_creat (file, mode);
-}
-#else
-extern int __libc_open (const char *file, int flags, mode_t mode);
-int creat (const char *file, mode_t mode)
-{
-	  return __libc_open (file, O_WRONLY|O_CREAT|O_TRUNC, mode);
-}
-#endif
-#endif
+// Implemented using open
 
 //#define __NR_link             9
 #ifdef L_link
@@ -134,16 +139,47 @@ _syscall1(int, unlink, const char *, pathname);
 #endif
 
 //#define __NR_execve           11
-#ifdef L_execve
+#ifdef L___syscall_execve
+#define __NR___syscall_execve __NR_execve
 #include <unistd.h>
-_syscall3(int, execve, const char *, filename, char *const *, argv,
-		  char *const *, envp);
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall3(int, __syscall_execve, const char *, filename, 
+		char *const *, argv, char *const *, envp);
+int execve(const char *filename, char *const *argv, char *const *envp)
+{ 
+	if (unlikely(filename==NULL || *filename=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(filename)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_execve(filename, argv, envp);
+}
 #endif
 
 //#define __NR_chdir            12
-#ifdef L_chdir
-#include <unistd.h>
-_syscall1(int, chdir, const char *, path);
+#ifdef L___syscall_chdir
+#define __NR___syscall_chdir __NR_chdir
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall1(int, __syscall_chdir, const char *, path);
+int chdir(const char *path)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_chdir(path);
+}
 #endif
 
 //#define __NR_time             13
@@ -173,8 +209,8 @@ time_t time (time_t *t)
 _syscall3(int, __syscall_mknod, const char *, path, __kernel_mode_t, mode, __kernel_dev_t, dev);
 int mknod(const char *path, mode_t mode, dev_t dev)
 { 
-	__kernel_dev_t k_dev;
 	/* We must convert the dev_t value to a __kernel_dev_t */
+	__kernel_dev_t k_dev;
 	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
 	return __syscall_mknod(path, mode, k_dev);
 }
@@ -207,6 +243,12 @@ static inline
 _syscall3(int, __syscall_lchown, const char *, path, __kernel_uid_t, owner, __kernel_gid_t, group);
 int lchown(const char *path, uid_t owner, gid_t group)
 {
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return __syscall_lchown(path, owner, group);
 }
 #endif
@@ -258,7 +300,7 @@ static inline
 _syscall1(int, __syscall_setuid, __kernel_uid_t, uid);
 int setuid(uid_t uid)
 {
-	if (uid == (uid_t) ~0 || uid != (uid_t) ((int) uid)) {
+	if (uid == (uid_t) ~0 || uid != (uid_t) ((__kernel_uid_t) uid)) {
 		__set_errno (EINVAL);
 		return -1;
 	}
@@ -267,19 +309,25 @@ int setuid(uid_t uid)
 #endif
 
 //#define __NR_getuid           24
-#ifdef L_getuid
+#ifdef L___syscall_getuid
 #include <unistd.h>
 #if defined (__alpha__)
 #define __NR_getuid     __NR_getxuid
 #endif
-_syscall0(uid_t, getuid);
+#define __NR___syscall_getuid __NR_getuid
+static inline 
+_syscall0(int, __syscall_getuid);
+uid_t getuid(void)
+{
+	return(__syscall_getuid());
+}
 #endif
 
 //#define __NR_stime            25
 #ifdef L_stime
 #include <time.h>
 #include <sys/time.h>
-#ifdef _NR_stime
+#ifdef __NR_stime
 _syscall1(int, stime, const time_t *, t);
 #else
 int stime(const time_t *when)
@@ -294,12 +342,7 @@ int stime(const time_t *when)
 #endif
 
 //#define __NR_ptrace           26
-#ifdef L___ptrace
-#include <sys/ptrace.h>
-#define __NR___ptrace __NR_ptrace
-_syscall4(long, __ptrace, enum __ptrace_request, request, __kernel_pid_t, pid,
-		void*, addr, void*, data);
-#endif
+//See ptrace.c
 
 
 //#define __NR_alarm            27
@@ -458,9 +501,30 @@ int kill(pid_t pid, int sig)
 #endif
 
 //#define __NR_rename           38
-#ifdef L_rename
+#ifdef L___syscall_rename
+#define __NR___syscall_rename __NR_rename
+#include <unistd.h>
+#include <string.h>
+#include <sys/param.h>
 #include <stdio.h>
-_syscall2(int, rename, const char *, oldpath, const char *, newpath);
+static inline
+_syscall2(int, __syscall_rename, const char *, oldpath, const char *, newpath);
+int rename(const char *oldpath, const char *newpath)
+{ 
+	if (unlikely((oldpath==NULL || newpath==NULL ||
+					*oldpath=='\0' || *newpath=='\0'))) 
+	{
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(oldpath)+1) >= NAME_MAX) || 
+				((strlen(newpath)+1) >= NAME_MAX))) 
+	{
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_rename(oldpath, newpath);
+}
 #endif
 
 //#define __NR_mkdir            39
@@ -511,26 +575,44 @@ static inline
 _syscall1(int, __syscall_setgid, __kernel_gid_t, gid);
 int setgid(gid_t gid)
 {
+	if (gid == (gid_t) ~0
+			|| gid != (gid_t) ((__kernel_gid_t) gid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setgid(gid));
 }
 #endif
 
 //#define __NR_getgid           47
-#ifdef L_getgid
+#ifdef L___syscall_getgid
 #include <unistd.h>
+#define __NR___syscall_getgid __NR_getgid
 #if defined (__alpha__)
 #define __NR_getgid     __NR_getxgid
 #endif
-_syscall0(gid_t, getgid);
+static inline 
+_syscall0(int, __syscall_getgid);
+gid_t getgid(void)
+{
+	return(__syscall_getgid());
+}
 #endif
 
 //#define __NR_signal           48
 
 //#define __NR_geteuid          49
-#ifdef	L_geteuid
+#ifdef	L___syscall_geteuid
 #include <unistd.h>
 #	ifdef	__NR_geteuid
-	_syscall0(uid_t, geteuid);
+#define __NR___syscall_geteuid __NR_geteuid
+	static inline 
+	_syscall0(int, __syscall_geteuid);
+	uid_t geteuid(void)
+	{
+		return(__syscall_geteuid());
+	}
 #	else
 	uid_t geteuid(void)
 	{
@@ -540,10 +622,16 @@ _syscall0(gid_t, getgid);
 #endif
 
 //#define __NR_getegid          50
-#ifdef	L_getegid
+#ifdef	L___syscall_getegid
 #include <unistd.h>
 #	ifdef	__NR_getegid
-	_syscall0(gid_t, getegid);
+#define __NR___syscall_getegid __NR_getegid
+static inline 
+_syscall0(int, __syscall_getegid);
+gid_t getegid(void)
+{
+	return(__syscall_getegid());
+}
 #	else
 	gid_t getegid(void)
 	{
@@ -588,24 +676,27 @@ _syscall3(int, __syscall_ioctl, int, fd, int, request, void *, arg);
 #endif
 
 //#define __NR_fcntl            55
-#ifdef L__fcntl
+#ifdef L___syscall_fcntl
 #include <stdarg.h>
 #include <fcntl.h>
-#define __NR__fcntl __NR_fcntl
-extern int _fcntl(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl, int, fd, int, cmd, long, arg);
-
-int __libc_fcntl(int fd, int command, ...)
+#define __NR___syscall_fcntl __NR_fcntl
+#ifdef __UCLIBC_HAS_LFS__
+static inline
+#endif
+_syscall3(int, __syscall_fcntl, int, fd, int, cmd, long, arg);
+int __libc_fcntl(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	if (cmd == F_GETLK64 || cmd == F_SETLK64 || cmd == F_SETLKW64) 
+	{
+			__set_errno(ENOSYS);
+			return -1;
+	}
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl(fd, command, arg);
+	return(__syscall_fcntl(fd, cmd, arg));
 }
 weak_alias(__libc_fcntl, fcntl)
 #endif
@@ -625,6 +716,7 @@ int setpgid(pid_t pid, pid_t pgid)
 #endif
 
 //#define __NR_ulimit           58
+//See ulimit.c
 
 //#define __NR_oldolduname      59
 
@@ -633,20 +725,50 @@ int setpgid(pid_t pid, pid_t pgid)
 #include <sys/stat.h>
 #define __NR___syscall_umask __NR_umask
 static inline 
-_syscall1(__kernel_mode_t, __syscall_umask, __kernel_mode_t, mask);
-mode_t umask(mode_t mask)
+_syscall1(__kernel_mode_t, __syscall_umask, __kernel_mode_t, mode);
+mode_t umask(mode_t mode)
 {
-	return(__syscall_umask(mask));
+	return(__syscall_umask(mode));
 }
 #endif
 
 //#define __NR_chroot           61
-#ifdef L_chroot
+#ifdef L___syscall_chroot
+#define __NR___syscall_chroot __NR_chroot
 #include <unistd.h>
-_syscall1(int, chroot, const char *, path);
+#include <string.h>
+#include <sys/param.h>
+static inline
+_syscall1(int, __syscall_chroot, const char *, path);
+int chroot(const char *path)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_chroot(path);
+}
 #endif
 
 //#define __NR_ustat            62
+#ifdef L___syscall_ustat
+#define __NR___syscall_ustat __NR_ustat
+#include <sys/ustat.h>
+static inline
+_syscall2(int, __syscall_ustat, unsigned short int, kdev_t, struct ustat *, ubuf);
+int ustat(dev_t dev, struct ustat *ubuf)
+{ 
+	/* We must convert the dev_t value to a __kernel_dev_t */
+	__kernel_dev_t k_dev;
+	k_dev = ((major(dev) & 0xff) << 8) | (minor(dev) & 0xff);
+	return __syscall_ustat(k_dev, ubuf);
+}
+#endif
+
 
 //#define __NR_dup2             63
 #ifdef L_dup2
@@ -702,6 +824,12 @@ static inline
 _syscall2(int, __syscall_setreuid, __kernel_uid_t, ruid, __kernel_uid_t, euid);
 int setreuid(uid_t ruid, uid_t euid)
 {
+	if (((ruid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((euid + 1) > (uid_t) ((__kernel_uid_t) -1U)))
+	{
+		__set_errno(EINVAL);
+		return -1;
+	}
 	return(__syscall_setreuid(ruid, euid));
 }
 #endif
@@ -714,6 +842,12 @@ static inline
 _syscall2(int, __syscall_setregid, __kernel_gid_t, rgid, __kernel_gid_t, egid);
 int setregid(gid_t rgid, gid_t egid)
 {
+	if (((rgid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((egid + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setregid(rgid, egid));
 }
 #endif
@@ -843,7 +977,7 @@ int getgroups(int n, gid_t *groups)
 		ngids = __syscall_getgroups(n, kernel_groups);
 		if (n != 0 && ngids > 0) {
 			for (i = 0; i < ngids; i++) {
-				(groups)[i] = kernel_groups[i];
+				groups[i] = kernel_groups[i];
 			}
 		}
 		return ngids;
@@ -994,6 +1128,12 @@ static inline
 _syscall3(int, __syscall_fchown, int, fd, __kernel_uid_t, owner, __kernel_gid_t, group);
 int fchown(int fd, uid_t owner, gid_t group)
 {
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_fchown(fd, owner, group));
 }
 #endif
@@ -1026,9 +1166,25 @@ _syscall3(int, setpriority, __priority_which_t, which, id_t, who, int, prio);
 //#define __NR_profil           98
 
 //#define __NR_statfs           99
-#ifdef L_statfs
+#ifdef L___syscall_statfs
+#define __NR___syscall_statfs __NR_statfs
+#include <string.h>
+#include <sys/param.h>
 #include <sys/vfs.h>
-_syscall2(int, statfs, const char *, path, struct statfs *, buf);
+static inline
+_syscall2(int, __syscall_statfs, const char *, path, struct statfs *, buf);
+int statfs(const char *path, struct statfs *buf)
+{ 
+	if (unlikely(path==NULL || *path=='\0')) {
+		__set_errno(EINVAL);
+		return -1;
+	}
+	if (unlikely(((strlen(path)+1) > NAME_MAX))) {
+		__set_errno(ENAMETOOLONG);
+		return -1;
+	}
+	return __syscall_statfs(path, buf);
+}
 #endif
 
 //#define __NR_fstatfs          100
@@ -1257,6 +1413,7 @@ weak_alias(modify_ldt, __modify_ldt);
 #include <sys/timex.h>
 _syscall1(int, adjtimex, struct timex *, buf);
 weak_alias(adjtimex, __adjtimex);
+weak_alias(adjtimex, ntp_adjtime);
 #endif
 
 //#define __NR_mprotect         125
@@ -1338,8 +1495,16 @@ _syscall2(int, bdflush, int, __func, long int, __data);
 #endif
 
 //#define __NR_sysfs            135
+//_syscall3(int, sysfs, int, option, unsigned int, index, char addr);
+
 
 //#define __NR_personality      136
+#ifdef __NR_personality
+#ifdef L_personality
+#include <sys/personality.h>
+_syscall1(int, personality, unsigned long int, __persona);
+#endif
+#endif
 
 //#define __NR_afs_syscall      137
 
@@ -1352,6 +1517,11 @@ static inline
 _syscall1(int, __syscall_setfsuid, __kernel_uid_t, uid);
 int setfsuid(uid_t uid)
 {
+	if (uid != (uid_t) ((__kernel_uid_t) uid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setfsuid(uid));
 }
 #endif
@@ -1366,6 +1536,11 @@ static inline
 _syscall1(int, __syscall_setfsgid, __kernel_gid_t, gid);
 int setfsgid(gid_t gid)
 {
+	if (gid != (gid_t) ((__kernel_gid_t) gid))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setfsgid(gid));
 }
 #endif
@@ -1391,9 +1566,15 @@ weak_alias(_newselect, select);
 #endif
 
 //#define __NR_flock            143
-#ifdef L_flock
+#ifdef L___syscall_flock
 #include <sys/file.h>
-_syscall2(int,flock,int,fd, int,operation);
+#define __NR___syscall_flock __NR_flock
+static inline
+_syscall2(int, __syscall_flock, int, fd, int, operation);
+int flock(int fd, int operation)
+{
+	return(__syscall_flock(fd, operation));
+}
 #endif
 
 //#define __NR_msync            144
@@ -1442,7 +1623,15 @@ _syscall1(int, fdatasync, int, fd);
 //#define __NR__sysctl          149
 #ifdef __NR__sysctl
 #ifdef L__sysctl
-#include <linux/sysctl.h>
+struct __sysctl_args {
+	int *name;
+	int nlen;
+	void *oldval;
+	size_t *oldlenp;
+	void *newval;
+	size_t newlen;
+	unsigned long __unused[4];
+};
 _syscall1(int, _sysctl, struct __sysctl_args *, args);
 int sysctl(int *name, int nlen, void *oldval, size_t *oldlenp,
 			void *newval, size_t newlen)
@@ -1611,6 +1800,13 @@ static inline
 _syscall3(int, __syscall_setresuid, __kernel_uid_t, rgid, __kernel_uid_t, egid, __kernel_uid_t, sgid);
 int setresuid(uid_t ruid, uid_t euid, uid_t suid)
 {
+	if (((ruid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((euid + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((suid + 1) > (uid_t) ((__kernel_uid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setresuid(ruid, euid, suid));
 }
 #endif
@@ -1655,9 +1851,14 @@ int getresuid (uid_t *ruid, uid_t *euid, uid_t *suid)
 #endif	
 
 //#define __NR_poll                     168
-#if defined(L_poll) && defined(__NR_poll) /* uClinux 2.0 doesn't have poll */
+#ifdef L_poll
+#ifdef __NR_poll
 #include <sys/poll.h>
 _syscall3(int, poll, struct pollfd *, fds, unsigned long int, nfds, int, timeout);
+#else
+/* uClinux 2.0 doesn't have poll, emulate it using select */
+#include "poll.c"
+#endif
 #endif
 
 //#define __NR_nfsservctl               169
@@ -1671,6 +1872,13 @@ static inline
 _syscall3(int, __syscall_setresgid, __kernel_gid_t, rgid, __kernel_gid_t, egid, __kernel_gid_t, sgid);
 int setresgid(gid_t rgid, gid_t egid, gid_t sgid)
 {
+	if (((rgid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((egid + 1) > (gid_t) ((__kernel_gid_t) -1U))
+			|| ((sgid + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_setresgid(rgid, egid, sgid));
 }
 #endif
@@ -1698,6 +1906,13 @@ int getresgid(gid_t *rgid, gid_t *egid, gid_t *sgid)
 #endif
 
 //#define __NR_prctl                    172
+#ifdef __NR_prctl
+#ifdef L_prctl
+#include <stdarg.h>
+//#include <sys/prctl.h>
+_syscall5(int, prctl, int, a, int, b, int, c, int, d, int, e);
+#endif
+#endif
 
 //#define __NR_rt_sigreturn             173
 //#define __NR_rt_sigaction             174
@@ -1813,6 +2028,12 @@ static inline
 _syscall3(int, __syscall_chown, const char *, path, __kernel_uid_t, owner, __kernel_gid_t, group);
 int chown(const char * path, uid_t owner, gid_t group)
 {
+	if (((owner + 1) > (uid_t) ((__kernel_uid_t) -1U))
+			|| ((group + 1) > (gid_t) ((__kernel_gid_t) -1U)))
+	{
+		__set_errno (EINVAL);
+		return -1;
+	}
 	return(__syscall_chown(path, owner, group));
 }
 #endif
@@ -1897,7 +2118,7 @@ int getrlimit (__rlimit_resource_t resource, struct rlimit *rlimits)
 
 //#define __NR_stat64             195
 #ifdef L___syscall_stat64
-#if defined __NR_stat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_stat64
 #define __NR___syscall_stat64 __NR_stat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1919,7 +2140,7 @@ int stat64(const char * file_name, struct stat64 * buf)
 
 //#define __NR_lstat64            196
 #ifdef L___syscall_lstat64
-#if defined __NR_lstat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_lstat64
 #define __NR___syscall_lstat64 __NR_lstat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1941,7 +2162,7 @@ int lstat64(const char * file_name, struct stat64 * buf)
 
 //#define __NR_fstat64            197
 #ifdef L___syscall_fstat64
-#if defined __NR_fstat64 && defined __UCLIBC_HAS_LFS__
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_fstat64
 #define __NR___syscall_fstat64 __NR_fstat64
 #include <unistd.h>
 #include <sys/stat.h>
@@ -1990,48 +2211,39 @@ _syscall2(int, pivot_root, const char *, new_root, const char *, put_old);
 
 //#define __NR_mincore		218
 //#define __NR_madvise		219
+#ifdef __NR_madvise
+#ifdef L_madvise
+_syscall3(int, madvise, void*, __addr, size_t, __len, int, __advice);
+#endif
+#endif
+
 //#define __NR_madvise1		219	/* delete when C lib stub is removed */
 
 //#define __NR_getdents64		220
 // See getdents64.c
 
 //#define __NR_fcntl64		221
-#ifdef L__fcntl64
-#ifdef __UCLIBC_HAS_LFS__
+#ifdef L___syscall_fcntl64
 #include <stdarg.h>
 #include <fcntl.h>
-#ifdef __NR_fcntl64
-#define __NR__fcntl64 __NR_fcntl64
-extern int _fcntl64(int fd, int cmd, long arg);
-
-_syscall3(int, _fcntl64, int, fd, int, cmd, long, arg);
-
-int fcntl64(int fd, int command, ...)
-{
-	long arg;
-	va_list list;
-
-	va_start(list, command);
-	arg = va_arg(list, long);
-
-	va_end(list);
-	return _fcntl64(fd, command, arg);
-}
+#if defined __UCLIBC_HAS_LFS__ && defined __NR_fcntl64
+#define __NR___syscall_fcntl64 __NR_fcntl64
+static inline
+_syscall3(int, __syscall_fcntl64, int, fd, int, cmd, long, arg);
 #else
-extern int _fcntl(int fd, int cmd, long arg);
-int fcntl64(int fd, int command, ...)
+#define __syscall_fcntl64  __syscall_fcntl
+extern int __syscall_fcntl(int fd, int cmd, ...);
+#endif
+int __libc_fcntl64(int fd, int cmd, ...)
 {
 	long arg;
 	va_list list;
-
-	va_start(list, command);
+	va_start(list, cmd);
 	arg = va_arg(list, long);
-
 	va_end(list);
-	return _fcntl(fd, command, arg);
+	return(__syscall_fcntl64(fd, cmd, arg));
 }
-#endif
-#endif
+weak_alias(__libc_fcntl64, fcntl64)
 #endif
 
 //#define __NR_security		223	/* syscall for security modules */
